@@ -1,36 +1,75 @@
+#
+# This file is part of MooseX-Traits-Attribute-MergeHashRef
+#
+# This software is Copyright (c) 2011 by Moritz Onken.
+#
+# This is free software, licensed under:
+#
+#   The (three-clause) BSD License
+#
 package MooseX::Traits::Attribute::MergeHashRef;
-our $VERSION = '1.001';
-
-use base 'Moose::Meta::Method::Accessor';
+BEGIN {
+  $MooseX::Traits::Attribute::MergeHashRef::VERSION = '1.002';
+}
+use Moose;
+extends 'Moose::Meta::Method::Accessor';
 
 use warnings;
 use strict;
+use Try::Tiny;
+use Carp;
 
 use Hash::Merge qw(merge);
 
 sub _inline_pre_body {
     my $self = shift;
     my $isa  = $self->associated_attribute->{isa};
-    unless ($isa eq 'HashRef') {
+    unless ( $isa eq 'HashRef' ) {
         warn 'MergeHashRef work on HashRef attributes only';
         return '';
     }
-    my $inv         = '$_[0]';
+    my $inv = '$_[0]';
 
     my $attr = $self->associated_attribute;
 
     my $mi = $attr->associated_class->get_meta_instance;
-    my $pred = $mi->inline_is_slot_initialized($inv, $attr->name);
-    my $old = 'my ($old) = '
-            . $pred . q{ ? }
-            . $self->_inline_get($inv) . q{ : ()} . ";\n";
+    my $pred = $mi->inline_is_slot_initialized( $inv, $attr->name );
+    my $old =
+        'my ($old) = ' 
+      . $pred . q{ ? }
+      . $self->_inline_get($inv)
+      . q{ : ()} . ";\n";
     return $old . q{
 use Hash::Merge qw(merge);
 $_[1] = merge ($_[1], $old) if(defined $old && defined $_[1]);
-
 };
 }
 
+override _generate_accessor_method_inline => sub {
+    my $self = shift;
+    my $attr = $self->associated_attribute;
+
+    return try {
+        $self->_compile_code(
+                              [ 'sub {',
+                                'if (@_ > 1) {',
+                                'use Hash::Merge qw(merge);',
+                                '$_[1] = merge ($_[1], ',
+                                $attr->_inline_instance_get('$_[0]'),
+                                ') if(defined ',
+                                $attr->_inline_instance_get('$_[0]'),
+                                ' && defined $_[1]);',
+                                $attr->_inline_set_value( '$_[0]', '$_[1]' ),
+                                '}',
+                                $attr->_inline_get_value('$_[0]'),
+                                '}',
+                              ] );
+    }
+    catch {
+        confess "Could not generate inline accessor because : $_";
+    };
+} unless( Moose->VERSION < 1.9);
+
 1;
 
 
@@ -43,15 +82,7 @@ MooseX::Traits::Attribute::MergeHashRef
 
 =head1 VERSION
 
-version 1.001
-
-=head1 NAME
-
-MooseX::Traits::Attribute::MergeHashRef - Merging HashRef attribute
-
-=head1 VERSION
-
-version 1.001
+version 1.002
 
 =head1 SYNOPSIS
 
@@ -75,13 +106,17 @@ This trait will merge values added to a HashRef attribute. It uses L<Hash::Merge
 The method  C<set_$attr> which resets the attribute with a given hashref is also created.
 Call C<clear_$attr> to clear the attribute.
 
+=head1 NAME
+
+MooseX::Traits::Attribute::MergeHashRef - Merging HashRef attribute
+
 =head1 AUTHOR
 
 Moritz Onken
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2010 by Moritz Onken.
+This software is Copyright (c) 2011 by Moritz Onken.
 
 This is free software, licensed under:
 
